class AuthMixin:
    def post_login(self, username=None, password=None):
        """``POST``s a login request."""
        if username is not None and password is not None:
            data = {"username": username, "password": password, "loginMode": 1}
        else:
            data = {"loginMode": 8}
        login_response = self.post("auth/login", json=data)
        if login_response.status_code != 204:
            login_response.raise_for_status()
        return login_response

    def post_logout(self):
        logout_response = self.post("auth/logout")
        if logout_response.status_code == 204:
            self.destroy_auth_token()

    # "Friendly" method aliases
    def login(self, username: str = None, password: str = None):
        """Logs in to MicroStrategy REST API.
        
        These credentials must be using MicroStrategy's standard authentication. 
        
        If no credentials are provided, the session will attempt to establish an anonymous connection.

        Returns:
            A ``requests`` response object with result of the login request
        """
        return self.post_login(username, password)

    def logout(self):
        """Closes the REST API session associated with the session object."""
        return self.post_logout()



from .utils import check_valid_session


class SessionsMixin:
    @check_valid_session
    def put_sessions(self):
        return self.put("sessions")

    @check_valid_session
    def get_sessions_userinfo(self):
        return self.get("sessions/userInfo")

    @check_valid_session
    def get_sessions(self):
        return self.get("sessions")

    # "Friendly" method aliases
    def extend_session(self):
        return self.put_sessions()

    def get_userinfo(self):
        return self.get_sessions_userinfo()

    def get_session_info(self):
        return self.get_sessions()



from .exceptions import SessionException

import json

from requests.utils import dict_from_cookiejar, cookiejar_from_dict


class SessionPersistenceMixin:
    def json(self):
        return json.dumps(
            {
                "base_url": self.base_url,
                "cookies": dict_from_cookiejar(self.cookies),
                "headers": self.headers,
            }
        )

    def update_from_json(self, data):
        if type(data) is dict:
            input_data = data
        elif type(data) is str:
            input_data = json.loads(data)
        else:
            input_data = dict()

        try:
            self.base_url = input_data["base_url"]
            self.cookies = cookiejar_from_dict(input_data["cookies"])
            self.headers.update(input_data["headers"])
        except KeyError as e:
            raise SessionException(e)



from .session import MSTRRESTSession


class AuthenticatedMSTRRESTSession(MSTRRESTSession):
    """A context manager for sessions interacting with the MicroStrategy REST API."""

    def __init__(self, base_url, username=None, password=None):
        super(AuthenticatedMSTRRESTSession, self).__init__(base_url)
        self._username = username
        self._password = password

    def __enter__(self):
        self.login(self._username, self._password)
        return self

    def __exit__(self, t, v, tb):
        self.logout()
